import json
from importlib import resources

import requests

import bscscan
from bscscan import configs
from bscscan.enums.fields_enum import FieldsEnum as fields
from bscscan.utils.parsing import ResponseParser as parser


class BscScan:
    def __new__(cls, api_key: str):
        with resources.path(configs, "stable.json") as path:
            config_path = str(path)
        return cls.from_config(api_key=api_key, config_path=config_path)

    @staticmethod
    def __load_config(config_path: str) -> dict:
        with open(config_path, "r") as f:
            return json.load(f)

    @staticmethod
    def __run(func, api_key: str):
        def wrapper(*args, **kwargs):
            url = (
                f"{fields.PREFIX}"
                f"{func(*args, **kwargs)}"
                f"{fields.API_KEY}"
                f"{api_key}"
            )
            r = requests.get(url)
            return parser.parse(r)

        return wrapper

    @classmethod
    def from_config(cls, api_key: str, config_path: str):
        config = cls.__load_config(config_path)
        for func, v in config.items():
            if not func.startswith("_"):  # disabled if _
                attr = getattr(getattr(bscscan, v["module"]), func)
                setattr(cls, func, cls.__run(attr, api_key))
        return cls



from .bscscan import BscScan
from .modules.accounts import Accounts as accounts
from .modules.blocks import Blocks as blocks
from .modules.contracts import Contracts as contracts
from .modules.proxy import Proxy as proxy
from .modules.stats import Stats as stats
from .modules.tokens import Tokens as tokens
from .modules.transactions import Transactions as transactions



from bscscan.enums.actions_enum import ActionsEnum as actions
from bscscan.enums.fields_enum import FieldsEnum as fields
from bscscan.enums.modules_enum import ModulesEnum as modules


class Stats:
    @staticmethod
    def get_total_bnb_supply() -> str:
        url = (
            f"{fields.MODULE}"
            f"{modules.STATS}"
            f"{fields.ACTION}"
            f"{actions.BNB_SUPPLY}"
        )
        return url

    @staticmethod
    def get_validators_list() -> str:
        url = (
            f"{fields.MODULE}"
            f"{modules.STATS}"
            f"{fields.ACTION}"
            f"{actions.VALIDATORS}"
        )
        return url



from bscscan.enums.actions_enum import ActionsEnum as actions
from bscscan.enums.fields_enum import FieldsEnum as fields
from bscscan.enums.modules_enum import ModulesEnum as modules
from bscscan.enums.tags_enum import TagsEnum as tags


class Proxy:
    @staticmethod
    def get_proxy_block_number() -> str:
        url = (
            f"{fields.MODULE}"
            f"{modules.PROXY}"
            f"{fields.ACTION}"
            f"{actions.ETH_BLOCK_NUMBER}"
        )
        return url

    @staticmethod
    def get_proxy_block_by_number(tag: str) -> str:
        url = (
            f"{fields.MODULE}"
            f"{modules.PROXY}"
            f"{fields.ACTION}"
            f"{actions.ETH_GET_BLOCK_BY_NUMBER}"
            f"{fields.TAG}"
            f"{tag}"
            f"{fields.BOOLEAN}"
            f"true"
        )
        return url

    @staticmethod
    def get_proxy_block_transaction_count_by_number(tag: str) -> str:
        url = (
            f"{fields.MODULE}"
            f"{modules.PROXY}"
            f"{fields.ACTION}"
            f"{actions.ETH_GET_BLOCK_TRANSACTION_COUNT_BY_NUMBER}"
            f"{fields.TAG}"
            f"{tag}"
        )
        return url

    @staticmethod
    def get_proxy_transaction_by_hash(txhash: str) -> str:
        url = (
            f"{fields.MODULE}"
            f"{modules.PROXY}"
            f"{fields.ACTION}"
            f"{actions.ETH_GET_TRANSACTION_BY_HASH}"
            f"{fields.TXHASH}"
            f"{txhash}"
        )
        return url

    @staticmethod
    def get_proxy_transaction_by_block_number_and_index(tag: str, index: str) -> str:
        url = (
            f"{fields.MODULE}"
            f"{modules.PROXY}"
            f"{fields.ACTION}"
            f"{actions.ETH_GET_TRANSACTION_BY_BLOCK_NUMBER_AND_INDEX}"
            f"{fields.TAG}"
            f"{tag}"
            f"{fields.INDEX}"
            f"{index}"
        )
        return url

    @staticmethod
    def get_proxy_transaction_count(address: str) -> str:
        url = (
            f"{fields.MODULE}"
            f"{modules.PROXY}"
            f"{fields.ACTION}"
            f"{actions.ETH_GET_TRANSACTION_COUNT}"
            f"{fields.ADDRESS}"
            f"{address}"
            f"{fields.TAG}"
            f"{tags.LATEST}"
        )
        return url

    @staticmethod
    def get_proxy_transaction_receipt(txhash: str) -> str:
        url = (
            f"{fields.MODULE}"
            f"{modules.PROXY}"
            f"{fields.ACTION}"
            f"{actions.ETH_GET_TRANSACTION_RECEIPT}"
            f"{fields.TXHASH}"
            f"{txhash}"
        )
        return url

    @staticmethod
    def get_proxy_call(to: str, data: str) -> str:
        url = (
            f"{fields.MODULE}"
            f"{modules.PROXY}"
            f"{fields.ACTION}"
            f"{actions.ETH_CALL}"
            f"{fields.TO}"
            f"{to}"
            f"{fields.DATA}"
            f"{data}"
            f"{fields.TAG}"
            f"{tags.LATEST}"
        )
        return url

    @staticmethod
    def get_proxy_code_at(address: str) -> str:
        url = (
            f"{fields.MODULE}"
            f"{modules.PROXY}"
            f"{fields.ACTION}"
            f"{actions.ETH_GET_CODE}"
            f"{fields.ADDRESS}"
            f"{address}"
            f"{fields.TAG}"
            f"{tags.LATEST}"
        )
        return url

    @staticmethod
    def get_proxy_storage_position_at(position: str, address: str) -> str:
        url = (
            f"{fields.MODULE}"
            f"{modules.PROXY}"
            f"{fields.ACTION}"
            f"{actions.ETH_GET_STORAGE_AT}"
            f"{fields.ADDRESS}"
            f"{address}"
            f"{fields.POSITION}"
            f"{position}"
            f"{fields.TAG}"
            f"{tags.LATEST}"
        )
        return url

    @staticmethod
    def get_proxy_gas_price() -> str:
        # NOTE: Results are in WEI
        url = (
            f"{fields.MODULE}"
            f"{modules.PROXY}"
            f"{fields.ACTION}"
            f"{actions.ETH_GAS_PRICE}"
        )
        return url

    @staticmethod
    def get_proxy_est_gas(
        from_addr: str, to_addr: str, data: str, value: str, gas_price: str, gas: str
    ) -> str:
        url = (
            f"{fields.MODULE}"
            f"{modules.PROXY}"
            f"{fields.ACTION}"
            f"{actions.ETH_ESTIMATE_GAS}"
            f"{fields.DATA}"
            f"{data}"
            f"{fields.FROM}"
            f"{from_addr}"
            f"{fields.TO}"
            f"{to_addr}"
            f"{fields.VALUE}"
            f"{value}"
            f"{fields.GAS_PRICE}"
            f"{gas_price}"
            f"{fields.GAS}"
            f"{gas}"
        )
        return url



from dataclasses import dataclass


@dataclass(frozen=True)
class FieldsEnum:
    ACTION: str = "&action="
    ADDRESS: str = "&address="
    API_KEY: str = "&apikey="
    BLOCK_TYPE: str = "&blocktype="
    BLOCKNO: str = "&blockno="
    BOOLEAN: str = "&boolean="
    CLIENT_TYPE: str = "&clienttype="
    CLOSEST: str = "&closest="
    CONTRACT_ADDRESS: str = "&contractaddress="
    DATA: str = "&data="
    END_BLOCK: str = "&endblock="
    END_DATE: str = "&enddate="
    FROM: str = "&from="
    GAS_PRICE: str = "&gasPrice="
    GAS: str = "&gas="
    HEX: str = "&hex="
    INDEX: str = "&index="
    MODULE: str = "module="
    OFFSET: str = "&offset="
    PAGE: str = "&page="
    POSITION: str = "&position="
    PREFIX: str = "https://api.bscscan.com/api?"
    SORT: str = "&sort="
    START_BLOCK: str = "&startblock="
    START_DATE: str = "&startdate="
    SYNC_MODE: str = "&syncmode="
    TAG: str = "&tag="
    TIMESTAMP: str = "&timestamp="
    TO: str = "&to="
    TXHASH: str = "&txhash="
    VALUE: str = "&value="



from functools import reduce
from typing import List

from bscscan.enums.actions_enum import ActionsEnum as actions
from bscscan.enums.fields_enum import FieldsEnum as fields
from bscscan.enums.modules_enum import ModulesEnum as modules
from bscscan.enums.tags_enum import TagsEnum as tags


class Accounts:
    @staticmethod
    def get_bnb_balance(address: str) -> str:
        url = (
            f"{fields.MODULE}"
            f"{modules.ACCOUNT}"
            f"{fields.ACTION}"
            f"{actions.BALANCE}"
            f"{fields.ADDRESS}"
            f"{address}"
            f"{fields.TAG}"
            f"{tags.LATEST}"
        )
        return url
        # r = requests.get(url)
        # return conversions.to_ticker_unit(parser.get_result(r))

    @staticmethod
    def get_bnb_balance_multiple(addresses: List[str]) -> str:
        # NOTE: Max 20 wallets at a time
        address_list = reduce(lambda w1, w2: str(w1) + "," + str(w2), addresses)
        url = (
            f"{fields.MODULE}"
            f"{modules.ACCOUNT}"
            f"{fields.ACTION}"
            f"{actions.BALANCE_MULTI}"
            f"{fields.ADDRESS}"
            f"{address_list}"
            f"{fields.TAG}"
            f"{tags.LATEST}"
        )
        return url
        # r = requests.get(url)
        # return [conversions.to_ticker_unit(r["balance"]) for r in parser.get_result(r)]

    @staticmethod
    def get_normal_txs_by_address(
        address: str,
        startblock: int,
        endblock: int,
        sort: str,
    ) -> str:
        # NOTE: Returns the last 10k events
        url = (
            f"{fields.MODULE}"
            f"{modules.ACCOUNT}"
            f"{fields.ACTION}"
            f"{actions.TXLIST}"
            f"{fields.ADDRESS}"
            f"{address}"
            f"{fields.START_BLOCK}"
            f"{str(startblock)}"
            f"{fields.END_BLOCK}"
            f"{str(endblock)}"
            f"{fields.SORT}"
            f"{sort}"
        )
        return url

    @staticmethod
    def get_normal_txs_by_address_paginated(
        address: str,
        page: int,
        offset: int,
        startblock: int,
        endblock: int,
        sort: str,
    ) -> str:
        url = (
            f"{fields.MODULE}"
            f"{modules.ACCOUNT}"
            f"{fields.ACTION}"
            f"{actions.TXLIST}"
            f"{fields.ADDRESS}"
            f"{address}"
            f"{fields.START_BLOCK}"
            f"{str(startblock)}"
            f"{fields.END_BLOCK}"
            f"{str(endblock)}"
            f"{fields.SORT}"
            f"{sort}"
            f"{fields.PAGE}"
            f"{str(page)}"
            f"{fields.OFFSET}"
            f"{str(offset)}"
        )
        return url

    @staticmethod
    def get_internal_txs_by_address(
        address: str,
        startblock: int,
        endblock: int,
        sort: str,
    ) -> str:
        # NOTE: Returns the last 10k events
        url = (
            f"{fields.MODULE}"
            f"{modules.ACCOUNT}"
            f"{fields.ACTION}"
            f"{actions.TXLIST_INTERNAL}"
            f"{fields.ADDRESS}"
            f"{address}"
            f"{fields.START_BLOCK}"
            f"{str(startblock)}"
            f"{fields.END_BLOCK}"
            f"{str(endblock)}"
            f"{fields.SORT}"
            f"{sort}"
        )
        return url

    @staticmethod
    def get_internal_txs_by_address_paginated(
        address: str,
        page: int,
        offset: int,
        startblock: int,
        endblock: int,
        sort: str,
    ) -> str:
        url = (
            f"{fields.MODULE}"
            f"{modules.ACCOUNT}"
            f"{fields.ACTION}"
            f"{actions.TXLIST_INTERNAL}"
            f"{fields.ADDRESS}"
            f"{address}"
            f"{fields.START_BLOCK}"
            f"{str(startblock)}"
            f"{fields.END_BLOCK}"
            f"{str(endblock)}"
            f"{fields.SORT}"
            f"{sort}"
            f"{fields.PAGE}"
            f"{str(page)}"
            f"{fields.OFFSET}"
            f"{str(offset)}"
        )
        return url

    @staticmethod
    def get_internal_txs_by_txhash(txhash: str) -> str:
        # NOTE: Returns the last 10k events
        url = (
            f"{fields.MODULE}"
            f"{modules.ACCOUNT}"
            f"{fields.ACTION}"
            f"{actions.TXLIST_INTERNAL}"
            f"{fields.TXHASH}"
            f"{txhash}"
        )
        return url

    @staticmethod
    def get_internal_txs_by_block_range_paginated(
        startblock: int,
        endblock: int,
        page: int,
        offset: int,
        sort: str,
    ) -> str:
        # NOTE: Returns the last 10k events
        url = (
            f"{fields.MODULE}"
            f"{modules.ACCOUNT}"
            f"{fields.ACTION}"
            f"{actions.TXLIST_INTERNAL}"
            f"{fields.START_BLOCK}"
            f"{str(startblock)}"
            f"{fields.END_BLOCK}"
            f"{str(endblock)}"
            f"{fields.SORT}"
            f"{sort}"
            f"{fields.PAGE}"
            f"{str(page)}"
            f"{fields.OFFSET}"
            f"{str(offset)}"
        )
        return url

    @staticmethod
    def get_bep20_token_transfer_events_by_address(
        address: str,
        startblock: int,
        endblock: int,
        sort: str,
    ) -> str:
        # NOTE: Returns the last 10k events
        url = (
            f"{fields.MODULE}"
            f"{modules.ACCOUNT}"
            f"{fields.ACTION}"
            f"{actions.TOKENTX}"
            f"{fields.ADDRESS}"
            f"{address}"
            f"{fields.START_BLOCK}"
            f"{str(startblock)}"
            f"{fields.END_BLOCK}"
            f"{str(endblock)}"
            f"{fields.SORT}"
            f"{sort}"
        )
        return url

    @staticmethod
    def get_bep20_token_transfer_events_by_contract_address_paginated(
        contract_address: str, page: int, offset: int, sort: str
    ) -> str:

        url = (
            f"{fields.MODULE}"
            f"{modules.ACCOUNT}"
            f"{fields.ACTION}"
            f"{actions.TOKENTX}"
            f"{fields.CONTRACT_ADDRESS}"
            f"{contract_address}"
            f"{fields.SORT}"
            f"{sort}"
            f"{fields.PAGE}"
            f"{str(page)}"
            f"{fields.OFFSET}"
            f"{str(offset)}"
        )
        return url

    @staticmethod
    def get_bep20_token_transfer_events_by_address_and_contract_paginated(
        contract_address: str, address: str, page: int, offset: int, sort: str
    ) -> str:

        url = (
            f"{fields.MODULE}"
            f"{modules.ACCOUNT}"
            f"{fields.ACTION}"
            f"{actions.TOKENTX}"
            f"{fields.CONTRACT_ADDRESS}"
            f"{contract_address}"
            f"{fields.ADDRESS}"
            f"{address}"
            f"{fields.SORT}"
            f"{sort}"
            f"{fields.PAGE}"
            f"{str(page)}"
            f"{fields.OFFSET}"
            f"{str(offset)}"
        )
        return url

    @staticmethod
    def get_validated_blocks_by_address(address: str) -> str:
        url = (
            f"{fields.MODULE}"
            f"{modules.ACCOUNT}"
            f"{fields.ACTION}"
            f"{actions.GET_MINED_BLOCKS}"
            f"{fields.ADDRESS}"
            f"{address}"
            f"{fields.BLOCK_TYPE}"
            f"blocks"
        )
        return url

    @staticmethod
    def get_validated_blocks_by_address_paginated(
        address: str, page: int, offset: int
    ) -> str:
        url = (
            f"{fields.MODULE}"
            f"{modules.ACCOUNT}"
            f"{fields.ACTION}"
            f"{actions.GET_MINED_BLOCKS}"
            f"{fields.ADDRESS}"
            f"{address}"
            f"{fields.BLOCK_TYPE}"
            f"blocks"
            f"{fields.PAGE}"
            f"{str(page)}"
            f"{fields.OFFSET}"
            f"{str(offset)}"
        )
        return url



import json
import os
import time
from datetime import datetime
from unittest import TestCase

from bscscan import BscScan

CONFIG_PATH = "bscscan/configs/stable.json"
API_KEY = os.environ["API_KEY"]


def load(fname):
    with open(fname, "r") as f:
        return json.load(f)


def dump(data, fname):
    with open(fname, "w") as f:
        json.dump(data, f, indent=2)


class Case(TestCase):
    _MODULE = ""

    def test_methods(self):
        print(f"\nMODULE: {self._MODULE}")
        config = load(CONFIG_PATH)
        bscscan = BscScan(API_KEY)
        for fun, v in config.items():
            if not fun.startswith("_"):  # disabled if _
                if v["module"] == self._MODULE:
                    res = getattr(bscscan, fun)(**v["kwargs"])
                    print(f"METHOD: {fun}, RTYPE: {type(res)}")
                    fname = f"logs/standard/{fun}.json"
                    log = {
                        "method": fun,
                        "module": v["module"],
                        "kwargs": v["kwargs"],
                        "log_timestamp": datetime.now().strftime("%Y-%m-%d-%H:%M:%S"),
                        "res": res,
                    }
                    dump(log, fname)
                    time.sleep(0.5)


class TestAccounts(Case):
    _MODULE = "accounts"


class TestBlocks(Case):
    _MODULE = "blocks"


class TestContracts(Case):
    _MODULE = "contracts"


class TestProxy(Case):
    _MODULE = "proxy"


class TestStats(Case):
    _MODULE = "stats"


class TestTokens(Case):
    _MODULE = "tokens"


class TestTransactions(Case):
    _MODULE = "transactions"


import requests


class ResponseParser:
    @staticmethod
    def parse(response: requests.Response):
        content = response.json()
        result = content["result"]
        if "status" in content.keys():
            status = bool(int(content["status"]))
            message = content["message"]
            assert status, f"{result} -- {message}"
        else:
            # GETH or Parity proxy msg format
            # TODO: see if we need those values
            jsonrpc = content["jsonrpc"]
            cid = int(content["id"])
        return result



from bscscan.enums.actions_enum import ActionsEnum as actions
from bscscan.enums.fields_enum import FieldsEnum as fields
from bscscan.enums.modules_enum import ModulesEnum as modules


class Transactions:
    @staticmethod
    def get_tx_receipt_status(txhash: str) -> str:
        url = (
            f"{fields.MODULE}"
            f"{modules.TRANSACTION}"
            f"{fields.ACTION}"
            f"{actions.GET_TX_RECEIPT_STATUS}"
            f"{fields.TXHASH}"
            f"{txhash}"
        )
        return url



from bscscan.enums.actions_enum import ActionsEnum as actions
from bscscan.enums.fields_enum import FieldsEnum as fields
from bscscan.enums.modules_enum import ModulesEnum as modules


class Blocks:
    @staticmethod
    def get_block_reward_by_block_number(block_no: str) -> str:
        url = (
            f"{fields.MODULE}"
            f"{modules.BLOCK}"
            f"{fields.ACTION}"
            f"{actions.GET_BLOCK_REWARD}"
            f"{fields.BLOCKNO}"
            f"{block_no}"
        )
        return url

    @staticmethod
    def get_est_block_countdown_time_by_block_number(block_no: str) -> str:
        url = (
            f"{fields.MODULE}"
            f"{modules.BLOCK}"
            f"{fields.ACTION}"
            f"{actions.GET_BLOCK_COUNTDOWN}"
            f"{fields.BLOCKNO}"
            f"{block_no}"
        )
        return url

    @staticmethod
    def get_block_number_by_timestamp(timestamp: int, closest: str) -> str:
        # NOTE: Supports UNIX timestamps in seconds
        url = (
            f"{fields.MODULE}"
            f"{modules.BLOCK}"
            f"{fields.ACTION}"
            f"{actions.GET_BLOCK_NUMBER_BY_TIME}"
            f"{fields.TIMESTAMP}"
            f"{timestamp}"
            f"{fields.CLOSEST}"
            f"{closest}"
        )
        return url



from dataclasses import dataclass


@dataclass(frozen=True)
class ModulesEnum:
    ACCOUNT: str = "account"
    BLOCK: str = "block"
    CONTRACT: str = "contract"
    GASTRACKER: str = "gastracker"
    PROXY: str = "proxy"
    STATS: str = "stats"
    TOKEN: str = "token"
    TRANSACTION: str = "transaction"




from dataclasses import dataclass


@dataclass(frozen=True)
class ActionsEnum:
    BALANCE_HISTORY: str = "balancehistory"
    BALANCE_MULTI: str = "balancemulti"
    BALANCE: str = "balance"
    BNB_SUPPLY: str = "bnbsupply"
    CHAIN_SIZE: str = "chainsize"
    DAILY_AVG_BLOCK_SIZE: str = "dailyavgblocksize"
    DAILY_AVG_BLOCK_TIME: str = "dailyavgblocktime"
    DAILY_AVG_GAS_LIMIT: str = "dailyavggaslimit"
    DAILY_AVG_GAS_PRICE: str = "dailyavggasprice"
    DAILY_AVG_HASH_RATE: str = "dailyavghashrate"
    DAILY_AVG_NET_DIFFICULTY: str = "dailyavgnetdifficulty"
    DAILY_BLK_COUNT: str = "dailyblkcount"
    DAILY_BLOCK_REWARDS: str = "dailyblockrewards"
    DAILY_GAS_USED: str = "dailygasused"
    DAILY_NET_UTILIZATION: str = "dailynetutilization"
    DAILY_NEW_ADDRESS: str = "dailynewaddress"
    DAILY_TX: str = "dailytx"
    DAILY_TXN_FEE: str = "dailytxnfee"
    DAILY_UNCLE_BLK_COUNT: str = "dailyuncleblkcount"
    ETH_BLOCK_NUMBER: str = "eth_blockNumber"
    ETH_CALL: str = "eth_call"
    ETH_DAILY_MARKET_CAP: str = "ethdailymarketcap"
    ETH_DAILY_PRICE: str = "ethdailyprice"
    ETH_ESTIMATE_GAS: str = "eth_estimateGas"
    ETH_GAS_PRICE: str = "eth_gasPrice"
    ETH_GET_BLOCK_BY_NUMBER: str = "eth_getBlockByNumber"
    ETH_GET_BLOCK_TRANSACTION_COUNT_BY_NUMBER: str = (
        "eth_getBlockTransactionCountByNumber"
    )
    ETH_GET_CODE: str = "eth_getCode"
    ETH_GET_STORAGE_AT: str = "eth_getStorageAt"
    ETH_GET_TRANSACTION_BY_BLOCK_NUMBER_AND_INDEX: str = (
        "eth_getTransactionByBlockNumberAndIndex"
    )
    ETH_GET_TRANSACTION_BY_HASH: str = "eth_getTransactionByHash"
    ETH_GET_TRANSACTION_COUNT: str = "eth_getTransactionCount"
    ETH_GET_TRANSACTION_RECEIPT: str = "eth_getTransactionReceipt"
    ETH_GET_UNCLE_BY_BLOCK_NUMBER_AND_INDEX: str = "eth_getUncleByBlockNumberAndIndex"
    ETH_PRICE: str = "ethprice"
    GAS_ESTIMATE: str = "gasestimate"
    GAS_ORACLE: str = "gasoracle"
    GET_ABI: str = "getabi"
    GET_BLOCK_COUNTDOWN: str = "getblockcountdown"
    GET_BLOCK_NUMBER_BY_TIME: str = "getblocknobytime"
    GET_BLOCK_REWARD: str = "getblockreward"
    GET_MINED_BLOCKS: str = "getminedblocks"
    GET_SOURCE_CODE: str = "getsourcecode"
    GET_STATUS: str = "getstatus"
    GET_TX_RECEIPT_STATUS: str = "gettxreceiptstatus"
    TOKEN_BALANCE_HISTORY: str = "tokenbalancehistory"
    TOKEN_BALANCE: str = "tokenbalance"
    TOKEN_CSUPPLY: str = "tokenCsupply"
    TOKEN_INFO: str = "tokeninfo"
    TOKEN_SUPPLY_HISTORY: str = "tokensupplyhistory"
    TOKEN_SUPPLY: str = "tokensupply"
    TOKENNFTTX: str = "tokennfttx"
    TOKENTX: str = "tokentx"
    TXLIST_INTERNAL: str = "txlistinternal"
    TXLIST: str = "txlist"
    VALIDATORS: str = "validators"



from setuptools import setup

setup(
    name="bscscan-python",
    version="1.0.0",
    description="A python API for bscscan.com.",
    url="https://github.com/pcko1/bscscan-python",
    author="Panagiotis-Christos Kotsias",
    author_email="kotsias.pan@gmail.com",
    license="MIT",
    packages=[
        "bscscan",
        "bscscan.configs",
        "bscscan.enums",
        "bscscan.modules",
        "bscscan.utils",
    ],
    install_requires=["requests"],
    include_package_data=True,
    zip_safe=False,
)



from bscscan.enums.actions_enum import ActionsEnum as actions
from bscscan.enums.fields_enum import FieldsEnum as fields
from bscscan.enums.modules_enum import ModulesEnum as modules


class Contracts:
    @staticmethod
    def get_contract_abi(address: str) -> str:
        url = (
            f"{fields.MODULE}"
            f"{modules.CONTRACT}"
            f"{fields.ACTION}"
            f"{actions.GET_ABI}"
            f"{fields.ADDRESS}"
            f"{address}"
        )
        return url

    @staticmethod
    def get_contract_source_code(address: str) -> str:
        url = (
            f"{fields.MODULE}"
            f"{modules.CONTRACT}"
            f"{fields.ACTION}"
            f"{actions.GET_SOURCE_CODE}"
            f"{fields.ADDRESS}"
            f"{address}"
        )
        return url



from decimal import Decimal


class Conversions:
    @staticmethod
    def to_ticker_unit(val: int, decimals: int = 18) -> Decimal:
        factor = Decimal("10") ** Decimal("-{}".format(decimals))
        return Decimal(val) * factor

    @staticmethod
    def to_smallest_unit(val: int, decimals: int = 18) -> Decimal:
        factor = Decimal("10") ** Decimal("+{}".format(decimals))
        return Decimal(val) * factor



from bscscan.enums.actions_enum import ActionsEnum as actions
from bscscan.enums.fields_enum import FieldsEnum as fields
from bscscan.enums.modules_enum import ModulesEnum as modules
from bscscan.enums.tags_enum import TagsEnum as tags


class Tokens:
    @staticmethod
    def get_total_supply_by_contract_address(contract_address: str) -> str:
        url = (
            f"{fields.MODULE}"
            f"{modules.STATS}"
            f"{fields.ACTION}"
            f"{actions.TOKEN_SUPPLY}"
            f"{fields.CONTRACT_ADDRESS}"
            f"{contract_address}"
        )
        return url

    @staticmethod
    def get_circulating_supply_by_contract_address(contract_address: str) -> str:
        url = (
            f"{fields.MODULE}"
            f"{modules.STATS}"
            f"{fields.ACTION}"
            f"{actions.TOKEN_CSUPPLY}"
            f"{fields.CONTRACT_ADDRESS}"
            f"{contract_address}"
        )
        return url

    @staticmethod
    def get_acc_balance_by_token_contract_address(
        contract_address: str, address: str
    ) -> str:
        url = (
            f"{fields.MODULE}"
            f"{modules.ACCOUNT}"
            f"{fields.ACTION}"
            f"{actions.TOKEN_BALANCE}"
            f"{fields.CONTRACT_ADDRESS}"
            f"{contract_address}"
            f"{fields.ADDRESS}"
            f"{address}"
            f"{fields.TAG}"
            f"{tags.LATEST}"
        )
        return url
